import { Component, Prop } from 'vue-property-decorator';
import { ZdComponent } from '@zeedhi/vuetify';
import ms from 'ms';
import { Timer as TimerClass } from './common/timer';

type Range = Array<number | null>;

type StylingType = {
  [key: string]: Range
};

@Component
export default class Timer extends ZdComponent {
  @Prop({ type: String }) public startTime!: string;

  @Prop({ type: Object }) public styling!: StylingType;

  @Prop({ type: String }) public defaultColor: string = 'info';

  @Prop({ type: String }) public rangeType!: 'percent' | 'value';

  @Prop({ type: String }) public tooltipPropName!: string;

  @Prop({ type: String }) public rangeCompare!: string;

  public instance!: TimerClass;

  private animationId!: number;

  public instanceType: typeof TimerClass = TimerClass;

  private updateTime() {
    const valueToCounter = (this.instance.getRow() || [])[this.startTime];
    if (this.instance.freezeTimer) {
      this.stopAnimation();
      return;
    }

    const [
      DAY,
      MONTH,
      YEAR,
      HOUR,
      MINUTE,
      SECOND,
    ] = valueToCounter.match(/\d+/g) || [];
    const dateInstance = new Date(YEAR, MONTH - 1, DAY, HOUR, MINUTE, SECOND);
    const elapsedTime = Date.now() - dateInstance.getTime();
    const date = new Date(elapsedTime);
    const hours = date.getUTCHours().toString().padStart(2, '0');
    const minutes = date.getUTCMinutes().toString().padStart(2, '0');
    const seconds = date.getUTCSeconds().toString().padStart(2, '0');
    const oneDay = ms('1d');
    this.instance.currentTime = `${hours}:${minutes}:${seconds}`;
    this.instance.elapsedTimeSeconds = (Number(hours) * 60 * 60) + (Number(minutes) * 60) + Number(seconds);
    if (elapsedTime > oneDay) {
      this.instance.setFixedTime('-');
      this.instance.freezeTimer = true;
    }
    this.animationId = this.startAnimationTime();
  }

  startAnimationTime(): number {
    if (!this.animationId || this.animationId === -1) {
      this.instance.startAnimationTime();
    }
    return requestAnimationFrame(this.updateTime);
  }

  inRangePercent(range: Range, curValue: number, valueToCompare: number): boolean {
    const [min, max] = range;
    const hasMin = typeof min === 'number';
    const hasMax = typeof max === 'number';
    const validMin = hasMin ? (curValue >= ((min || 0) / 100) * valueToCompare) : true;
    const validMax = hasMax ? (curValue <= ((max || 0) / 100) * valueToCompare) : true;
    return validMin && validMax;
  }

  inRangeValue(range: Range, curValue: number): boolean {
    const [min, max] = range;
    const hasMin = typeof min === 'number';
    const hasMax = typeof max === 'number';
    const validMin = hasMin ? (curValue >= (min || 0)) : true;
    const validMax = hasMax ? (curValue <= (max || 0)) : true;
    return validMin && validMax;
  }

  inRange(range: Range, curValue: number, valueToCompare = 0, type = 'value'): boolean | null {
    if (type === 'value') return this.inRange(range, curValue);
    if (type === 'percent') return this.inRangePercent(range, curValue, (valueToCompare || 0));
    return null;
  }

  // @TODO: Implement handler using timeFormat and dateFormat

  getColor(): string {
    const style = this.instance.styling || {};
    if (this.instance.freezeTimer) return 'defaultColorFreezeTime' || '';
    const colorToApply = Object.keys(style).find((color) => {
      const range = style[color] || [];
      const row = this.instance.getRow();
      if (!row) return this.defaultColor;
      const minutes = this.rangeCompare in row ? row[this.rangeCompare] : '';
      const secondsToCompare = (Number(minutes) * 60);
      const isValidRange = this.inRange(range, this.instance.elapsedTimeSeconds || 0, secondsToCompare, this.rangeType);
      return isValidRange;
    }) || this.defaultColor;
    return colorToApply;
  }

  stopAnimation() {
    cancelAnimationFrame(this.animationId);
    this.animationId = -1;
  }

  mounted() {
    if (this.$parent && (this.$parent as any).row) {
      const rowToSet = (this.$parent as any).row;
      this.instance.setRow(rowToSet);
      this.updateTime();
    }
  }
}
