import {
  AlertService,
  LoadingService, Modal, ModalService,
} from '@zeedhi/common';
import {
  Metadata, RestDatasource, Http,
} from '@zeedhi/core';
import { TekMemoryDatasource } from '@zeedhi/teknisa-components-common';
import { TekGrid } from '@zeedhi/teknisa-components-vuetify';
import { ZdTextInput, ZdTextarea } from '@zeedhi/vuetify';
import { Highlight } from '@zeedhi/zd-highlight';

type DataSourceTypes = (RestDatasource | TekMemoryDatasource);

type TekGridWithDataSource = TekGrid & {
  datasource?: DataSourceTypes
};

export class LogsController {
  private modalConfirmDeleteLog!: Modal;

  private modalLogs!: Modal;

  public async createModals() {
    this.modalConfirmDeleteLog = await ModalService.createFromJson('modal-confirm-delete-log', true);
    this.modalLogs = await ModalService.createFromJson('modal-logs', true);
  }

  public destroyModals() {
    this.modalConfirmDeleteLog?.destroy();
    this.modalLogs?.destroy();
  }

  public openModalConfirmDeleteLog() {
    this.modalConfirmDeleteLog.show();
  }

  public closeModalConfirmDeleteLog() {
    this.modalConfirmDeleteLog.hide();
  }

  public closeModalLogs() {
    this.modalLogs.hide();
  }

  public async openModalLogs({ row }: any) {
    let result;
    LoadingService.show('Carregando...');
    this.modalLogs.show();

    const {
      CDINTEGRACAOERR, CDFILIAL, NMINTEGRACAO, DTINCLUSAO,
    } = row;

    try {
      result = await Http.post('/order/logDetails', { CDINTEGRACAOERR });
      const jsonParse = JSON.parse(result.data.data);
      const jsonStringify = JSON.stringify(jsonParse, null, 2);
      const instance = Metadata.getInstance<Highlight>('logDetailHighlight');
      instance.code = jsonStringify;
    } catch (error) {
      const instance = Metadata.getInstance<Highlight>('logDetailHighlight');
      instance.code = 'Nenhum registro encontrado.';

      if (result.data.message) {
        AlertService.show({
          name: 'fail', text: result.data.message, right: true, top: true, color: 'red',
        });
      }
    } finally {
      LoadingService.hide();
    }

    const textarea = Metadata.getInstance<ZdTextarea>('logDetailTextarea');
    const replace = row.DSINTEGRACAOERR.replace(/\\n/g, '\n');
    textarea.value = replace;

    const textCdFilial = Metadata.getInstance<ZdTextInput>('textCdFilial');
    textCdFilial.value = CDFILIAL;

    const textNmIntegracao = Metadata.getInstance<ZdTextInput>('textNmIntegracao');
    textNmIntegracao.value = NMINTEGRACAO;

    const textDtInclusao = Metadata.getInstance<ZdTextInput>('textDtInclusao');
    textDtInclusao.value = DTINCLUSAO;
  }

  public getInstancesFromModalConfirmDeleteLog(row: any) {
    const container = this.modalConfirmDeleteLog.children.find((child: any) => child.name === 'confirmDeleteLogContainer');
    const form = container?.children?.find((child: any) => child.name === 'confirmDeleteform');

    form?.children?.map((child: any) => {
      child.value = row[child.name] || '';

      return child;
    });
  }

  public buttonDeleteLog({ event, row }: any) {
    event?.stopPropagation();
    this.openModalConfirmDeleteLog();
    this.getInstancesFromModalConfirmDeleteLog(row);
  }

  public getGrid() {
    const instance = Metadata.getInstance<TekGrid>('gridLogs');
    return instance;
  }

  public getIntegracaoErrFromModalConfirmDeleteLog() {
    const container = this.modalConfirmDeleteLog.children.find((child: any) => child.name === 'confirmDeleteLogContainer');
    const form = container?.children?.find((child: any) => child.name === 'confirmDeleteform');
    const textIntegracaoErr = form?.children?.find((child: any) => child.name === 'CDINTEGRACAOERR');
    const cdIntegracaoErr = textIntegracaoErr?.value;

    return cdIntegracaoErr;
  }

  public async deleteLogs(row: any) {
    try {
      const CDINTEGRACAOERR = this.getIntegracaoErrFromModalConfirmDeleteLog();
      const { data } = await Http.post('/order/deleteLogs', { CDINTEGRACAOERR });

      if (data.error) {
        throw new Error(`Falha ao excluir o log: ${data.message}`);
      }

      const grid: TekGridWithDataSource = this.getGrid();
      const findRowToDelete = (currentRow: any) => row.CDINTEGRACAOERR === currentRow.CDINTEGRACAOERR;

      const { data: gridData } = grid.datasource || {};

      if (Array.isArray(gridData)) {
        const idxRowToDelete = gridData.findIndex(findRowToDelete);

        gridData.splice(idxRowToDelete, 1);
        grid.datasource?.get();

        AlertService.show({
          name: 'deleteSuccess', text: 'Log excluído com sucesso!', top: true, color: 'green',
        });
      }
    } catch (error) {
      this.closeModalLogs();

      if (error instanceof Error) {
        AlertService.show({
          name: 'deleteFail', text: error.message, top: true, color: 'red',
        });
      }
    } finally {
      this.closeModalConfirmDeleteLog();
    }
  }
}
