import {
  Modal, ModalService, Form, LoadingService, Button, Chip, Tooltip,
  DateRange, Select, Footer, IGridEvent, ITabsBeforeChangeEventParam,
} from '@zeedhi/common';
import {
  Http, IDictionary, IEventParam, Metadata, dayjs,
} from '@zeedhi/core';
import { TekGrid, TekTreeGrid } from '@zeedhi/teknisa-components-common';
import Format from '@/helpers/Format';
import DateTime from '@/helpers/DateTime';
import Graph from '@/helpers/Graph';
import ms from 'ms';
import { alertService } from '@/helpers/AlertService';
import { ZdButton, ZdSelect } from '@zeedhi/vuetify';
import {
  Comanda, DataResponse, ParamsHistoryDetails, Instances,
  ParamsRequestPersistentDetails, RequestOrderItemsRepo,
  RowOpenPersistentDetailsItem, RequestOrderItemsRepoSub,
  Address,
} from '../types/InterfaceMainPageController';

export class MainPageController {
  private persistentModal!: Modal;

  private persistentDetailsItemModal!: Modal;

  private modalFilter!: Modal;

  private modalCancel!: Modal;

  private modalGraph!: Modal;

  private modalForward!: Modal;

  private modalScheduledFilter!: Modal;

  private currentInterval: number = 0;

  private activeTab: String = 'orders-tab';

  private firstTime = true;

  public async orderscreateModalMainAndFilter(): Promise<void> {
    this.persistentModal = await ModalService.createFromJson('main-page-persistent-modal', true);
    this.modalFilter = await ModalService.createFromJson('main-page-modal-filter', true);
    this.modalScheduledFilter = await ModalService.createFromJson('main-page-scheduled-filter', true);
  }

  public openPersistentModal({ column, row }: IEventParam<TekGrid>): void {
    switch (column.name) {
      case 'NRTELECONS':
        this.persistentModal.hide();
        break;

      case 'actions':
        this.persistentModal.hide();
        break;

      case 'actions-antecipation':
        this.persistentModal.hide();
        break;

      default:
        this.openOrderDetailsModal(row);
    }
  }

  public getGrid(): TekGrid {
    if (this.activeTab === 'orders-tab') {
      return Metadata.getInstance<TekGrid>('main-page-grid');
    }

    if (this.activeTab === 'scheduled-orders-tab') {
      return Metadata.getInstance<TekGrid>('scheduled-orders-grid');
    }

    throw new Error('Invalid tab name');
  }

  public async ordersGridMounted(): Promise<void> {
    await this.historyToday();
  }

  public setDefaultDate({ component }: IEventParam<DateRange>): void {
    const today = DateTime.now({ outputFormat: 'DD/MM/YYYY' });
    component.value = [today, today];
  }

  public setDefaultPickUpLocation({ component }: IEventParam<Select>): void {
    component.value = ['C', 'N', 'S'];
  }

  public formFilter(): Form | null {
    const formKey = this.activeTab === 'scheduled-orders-tab'
      ? 'main-page-scheduled-form' : 'main-page-modal-form';

    try {
      return Metadata.getInstance<Form>(formKey);
    } catch {
      return null;
    }
  }

  public validateForm(): boolean | null {
    const form = this.formFilter();
    const valid = form ? form.validate() : null;
    return valid;
  }

  public closePersistentModal(): void {
    this.persistentModal.destroy();
  }

  public closePersistentDetailsItemModal(): void {
    this.persistentDetailsItemModal.destroy();
  }

  public openModalFilter(): void {
    this.modalFilter.show();
  }

  public closeModalFilter(): void {
    this.modalFilter.hide();
  }

  public openModalScheduledFilter(): void {
    this.modalScheduledFilter.show();
  }

  public closeModalScheduledFilter(): void {
    this.modalScheduledFilter.hide();
  }

  public destroyModals(): void {
    this.persistentModal.destroy();
    this.modalFilter.destroy();
    this.modalScheduledFilter.destroy();
    if (this.modalGraph) {
      this.modalGraph.destroy();
    }
  }

  public async openModalGraph() {
    if (this.modalGraph) this.closeModalGraph();
    this.modalGraph = await ModalService.createFromJson('main-page-modal-graph', true);
    this.modalGraph.show();
    const tooltip: any = this.getGrid()?.toolbarSlot.find((element: any) => element.name === 'tooltip-start-and-stop');
    if (this.currentInterval > 0) this.stopRequest(tooltip);
  }

  public closeModalGraph(): void {
    this.modalGraph.destroy();
  }

  public filter(): void {
    const grid = this.getGrid();
    const form = Metadata.getInstance<Form>('main-page-form-filter');
    grid.datasource.setFilter(form.value);
  }

  public cleanFilter(): void {
    const form = this.formFilter();
    if (form && form !== null) {
      form.clearForm();
      form.value.localConsumption = 'C';
    }
  }

  public tooltipPhone({ row }: any): string {
    const phone = row.NRTELECONS !== null ? row.NRTELECONS : '';
    const regex = /(\d{2})(\d{5})(\d{4})/;
    const formatPhone = phone.replace(regex, '($1) $2-$3');
    return formatPhone;
  }

  public copyPhone({ row }: any): void {
    const phone = row.NRTELECONS;
    navigator.clipboard.writeText(phone);
    alertService('success', 'Telefone copiado', 'green');
  }

  public tooltipSynced({ row }: any): string {
    return row.IDSINCMSDE === 'S' ? 'Sim' : 'Não';
  }

  public btnSyncedIcon({ row }: any): string {
    return row.IDSINCMSDE === 'S' ? 'mdi-sync' : 'mdi-sync-off';
  }

  public btnSyncedColor({ row }: any): string {
    return row.IDSINCMSDE === 'S' ? 'green' : 'red';
  }

  public tooltipScheduled({ row }: any): string {
    return row.AGENDADO;
  }

  public btnScheduled({ row }: any): string {
    return row.AGENDADO === 'Sim' ? 'green' : 'red';
  }

  public labelStatus({ row }: any): string {
    return row.IDSTCOMANDA;
  }

  public tooltipModality({ row }: any): string {
    return row.TOGO;
  }

  public time({ row }: any): string {
    return row.TEMPOPEDIDO !== null ? `${row.TEMPOPEDIDO}m` : '';
  }

  public btnModalityIcon({ row }: any): string {
    switch (row.TOGO) {
      case 'DELIVERY':
        return 'mdi-bike-fast';

      case 'RETIRAR NO BALCÃO':
        return 'mdi-store';

      case 'CONSUMO NO LOCAL':
        return 'mdi-food-fork-drink';

      default:
        return '';
    }
  }

  public colorStatus({ component }: any): void {
    switch (component.label) {
      case 'cancelado':
        component.cssClass = 'canceled';
        break;

      case 'finalizado':
        component.cssClass = 'finished';
        break;

      case 'em produção':
        component.cssClass = 'production';
        break;

      case 'impresso':
        component.cssClass = 'printed';
        break;

      case 'agendado':
      case 'não sincronizado':
      case 'expedido':
      case 'não pago':
        component.cssClass = 'other';
        break;

      default:
        component.cssClass = 'default';
    }
  }

  public rowStyleConditions(row: IDictionary): Object {
    return {
      cssStyle: { color: row.IDSINCMSDE === 'N' ? 'red' : '' },
    };
  }

  public async historyToday(paramsFilter?: ParamsHistoryDetails): Promise<void> {
    const grid = this.getGrid();
    let idSincAgenda: string[] = [];
    let params;
    let dates;
    const dateTime = DateTime.now({ outputFormat: 'DD/MM/YYYY' });
    let dtInicial = `${dateTime} 00:00:00`; // `${dateTime} 00:00:00`;
    const dtFinal = `${dateTime} 23:59:59`;
    if (this.activeTab === 'orders-tab') {
      idSincAgenda = ['S'];
    } else if (this.activeTab === 'scheduled-orders-tab') {
      idSincAgenda = ['N'];
      dtInicial = `${dateTime} 00:00:00`;
    }
    if (paramsFilter !== undefined) {
      dates = { dtInicial: paramsFilter.dtInicial, dtFinal: paramsFilter.dtFinal };
      params = paramsFilter;
      params.idSincAgenda = idSincAgenda;
    } else {
      params = {
        dtInicial,
        dtFinal,
        idSincAgenda,
      };
      dates = { dtInicial: params.dtInicial, dtFinal: params.dtFinal };
    }

    try {
      LoadingService.show();
      const { data: { data, error, message } }: DataResponse<Comanda> = await Http.post('/order/history', params);

      if (error) throw new Error(message);

      const formatDataResult = this.treatData(data);
      grid.datasource.updateData(formatDataResult);
    } catch (error) {
      if (error instanceof Error) {
        grid.datasource.updateData([]);
        alertService('error', error.message, 'red');
      }
    } finally {
      LoadingService.hide();
      this.setPeriodFooter(dates);
    }
  }

  public sortFunction(value1: any, value2: any, columnName: string): boolean {
    const columns = ['CDFILIAL', 'NRCOMANDA', 'NRPEDCENTRAL', 'NRCOMANDAEXT', 'VRPEDIDO', 'QTDPROD'];
    if (columns.includes(columnName)) {
      const numericValue1 = Number(value1);
      const numericValue2 = Number(value2);

      if (!Number.isNaN(numericValue1) && !Number.isNaN(numericValue2)) {
        return numericValue1 > numericValue2;
      }
    }
    if (columnName === 'CDLOJA') {
      const numericValue1 = Number(value1.split(' ')[0]);
      const numericValue2 = Number(value2.split(' ')[0]);

      if (!Number.isNaN(numericValue1) && !Number.isNaN(numericValue2)) {
        return numericValue1 > numericValue2;
      }
    }
    if (columnName === 'DTHRABERMESA' || columnName === 'DTHRAGENDADA') {
      return (dayjs(value1, 'DD/MM/YYYY HH:mm:ss').isAfter(dayjs(value2, 'DD/MM/YYYY HH:mm:ss')));
    }
    return String(value1).toUpperCase() > String(value2).toUpperCase();
  }

  public onLoadTimer({ row, component }: any) {
    const orderBlockStatus = ['finalizado', 'cancelado', 'expedido', 'impresso'];
    if (row && component && typeof component.setFixedTime === 'function') {
      const freezeTimer = orderBlockStatus.includes(row.IDSTCOMANDA) || row.IDSINCMSDE === 'N';
      if (freezeTimer) component.setFixedTime('-');
    }
  }

  private getChildElement(component: Modal | Form | Tooltip, name: string) {
    const element = component?.children.find((child: any) => child.name === name);
    return element;
  }

  public async openModalCancelOrder({ row }: any) {
    if (this.modalCancel) this.closeModalCancelOrder();
    this.modalCancel = await ModalService.createFromJson('main-page-modal-cancel', true);
    const formModalCancel: any = this.getChildElement(this.modalCancel, 'main-page-modal-form-cancel');
    const branch: any = this.getChildElement(formModalCancel, 'form-branch');
    const order: any = this.getChildElement(formModalCancel, 'form-order');
    const orderFos: any = this.getChildElement(formModalCancel, 'form-order-fos');
    const externalNumber: any = this.getChildElement(formModalCancel, 'form-external-number');
    const origin: any = this.getChildElement(formModalCancel, 'form-origin');

    branch.text = `<strong>${branch.text}</strong> ${row.CDFILIAL}`;
    branch.value = row.CDFILIAL;

    order.text = `<strong>${order.text}</strong> ${row.NRCOMANDA}`;
    order.value = row.NRCOMANDA;

    orderFos.text = `<strong>${orderFos.text}</strong> ${row.NRVENDAREST}`;
    orderFos.value = row.NRVENDAREST;

    externalNumber.text = `<strong>${externalNumber.text}</strong> ${row.NRCOMANDAEXT}`;
    externalNumber.value = row.NRCOMANDAEXT;

    origin.text = `<strong>${origin.text}</strong> ${row.IDORGCMDVENDA}`;
    origin.value = row.IDORGCMDVENDA;

    formModalCancel.value = {
      branch: branch.value,
      order: order.value,
      orderFos: orderFos.value,
      externalNumber: externalNumber.value,
      origin: origin.value,
    };

    this.modalCancel.show();
  }

  private closeModalCancelOrder(): void {
    this.modalCancel?.destroy();
  }

  private getChipByKey(key: string): Chip | null {
    try {
      return Metadata.getInstance(`zdChipStatus_${key}`);
    } catch (err) {
      return null;
    }
  }

  private getButtonByKey(key: string): Button | null {
    try {
      if (this.activeTab === 'orders-tab') {
        return Metadata.getInstance(`buttonCancelOrder_${key}`);
      }
      if (this.activeTab === 'scheduled-orders-tab') {
        return Metadata.getInstance(`buttonCancelscheduledOrders_${key}`);
      }
    } catch (err) {
      return null;
    }

    return null;
  }

  public checkIsDisabled({ row }: any): boolean {
    const statusComanda = row.IDSTCOMANDA;
    const allowedValues = ['cancelado', 'impresso', 'expedido', 'finalizado'];
    return allowedValues.includes(statusComanda);
  }

  public async executeCancelOrder() {
    const formModalCancel = Metadata.getInstance<Form>('main-page-modal-form-cancel');
    if (!formModalCancel.validate()) return;
    await this.cancelOrder();
  }

  public customizeComponentChipAndLockCancel(order: string): void {
    const chip = this.getChipByKey(order);
    if (chip) this.colorStatus({ component: chip });

    const button = this.getButtonByKey(order);
    if (button) button.disabled = true;
  }

  public async cancelOrder(): Promise<void> {
    try {
      const formModalCancel: any = this.getChildElement(this.modalCancel, 'main-page-modal-form-cancel');
      const instanceForm = Metadata.getInstance<Form>(formModalCancel.name);

      if (!instanceForm.validate()) return;

      const {
        branch, order, orderFos, formCancellationReason,
      } = instanceForm.value;

      const CDOPERADOR = localStorage.getItem('ID');

      LoadingService.show();

      const { data: { message, error } } = await Http.post('/order/cancel', {
        CDFILIAL: branch,
        CDOPERADOR,
        NRCOMANDA: order,
        NRVENDAREST: orderFos,
        DSCANCMOTIV: formCancellationReason,
      });

      if (error) throw new Error(message);

      this.closeModalCancelOrder();

      const chip = this.getChipByKey(order);

      if (chip) {
        chip.label = 'cancelado';
        this.colorStatus({ component: chip });
      }

      alertService('alert', `${message}`, 'success');

      const paramsFilter = this.formatParamsFilter();
      await (paramsFilter ? this.historyToday(paramsFilter) : this.historyToday());
      this.customizeComponentChipAndLockCancel(order);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    } finally {
      LoadingService.hide();
    }
  }

  public filterClick({ event }: IGridEvent): void {
    event?.preventDefault();
    this.openModalFilter();
  }

  public formatParamsFilter(): ParamsHistoryDetails | null {
    const result = this.formFilter();
    if (result && result.value !== undefined) {
      const { value } = result;
      const dtIniAgendada = value.dtAgendada && value.dtAgendada.length > 0 ? `${value.dtAgendada[0]} 00:00:00` : '';
      const dtFimAgendada = value.dtAgendada && value.dtAgendada.length > 0 ? `${value.dtAgendada[1]} 23:59:59` : '';
      return {
        dtInicial: `${value.period[0]} 00:00:00`,
        dtFinal: `${value.period[1]} 23:59:59`,
        dtIniAgendada,
        dtFimAgendada,
        cdFilial: value.branch,
        nrComanda: value.order,
        clienteConsumidor: value.consumer,
        idOrgCmdVenda: value.origin,
        idStComanda: value.status,
        idSincAgenda: value.idSincAgenda,
        idSincMsde: value.notSynced === 'N' ? ['N', 'S'] : [value.notSynced],
        idRetBalLoja: value.pickUpLocation,
        ordersType: [],
        scheduledTab: [],
      };
    }
    return null;
  }

  public async startFilter(): Promise<void> {
    const paramsFilter = this.formatParamsFilter();
    const grid: any = this.getGrid();
    grid.appliedFilter = paramsFilter;
    if (paramsFilter) await this.executeStartFilter(paramsFilter);
  }

  public async executeStartFilter(paramsFilter: ParamsHistoryDetails): Promise<void> {
    try {
      if (!this.validateForm()) return;
      this.historyToday(paramsFilter);
      this.closeModalFilter();
      this.closeModalScheduledFilter();
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public treatData(data: any[]): Comanda[] {
    data.forEach((element: any) => {
      element.IDSTCOMANDA = Format.apply(element.IDSTCOMANDA, 'status', { isSincMSDE: element.IDSINCMSDE });
    });
    return data;
  }

  public setPeriodFooter(dates: any): void {
    const dtInicial = dates.dtInicial ? dates.dtInicial.split(' ')[0] : dates[0];
    const dtFinal = dates.dtFinal ? dates.dtFinal.split(' ')[0] : dates[1];

    const grid = this.getGrid();
    const footer = grid.getChildInstance<Footer>('footer-container');
    const footerText: any = footer.getChildInstance<Text>('period-field');
    footerText.text = `Período: ${dtInicial} - ${dtFinal}`;
  }

  public addGraphModalToolbar() {
    const grid = this.getGrid();

    const rightActions: any = grid.toolbarSlot.find((el: any) => el.tag === 'span' && el.component === 'ZdTag');
    const existsAction: boolean = rightActions && !!rightActions.children.find((el: any) => el.name === 'createGraphModal');
    if (rightActions && !existsAction) {
      rightActions.children.unshift(
        {
          name: 'createGraphModal',
          component: 'ZdTooltip',
          label: 'Gráficos',
          bottom: true,
          children: [
            {
              name: 'buttonOpenGraphModal',
              component: 'ZdButton',
              flat: true,
              icon: true,
              iconName: 'mdi-chart-pie',
              events: {
                click: () => {
                  this.openModalGraph();
                },
              },
            },
          ],
        },
      );
    }
    LoadingService.hide();
    this.modalCancel.hide();
  }

  public createGraphOrigin() {
    const grid = this.getGrid();

    const datasource: any[] = Object.entries((<any>grid.datasource).filteredData.reduce((acc: { [x: string]: number; },
      cv: { IDORGCMDVENDA: string | number; }) => {
      acc[cv.IDORGCMDVENDA] = acc[cv.IDORGCMDVENDA] || 0;
      acc[cv.IDORGCMDVENDA] += 1;
      return acc;
    }, {}));

    datasource.forEach((element: any) => {
      element[0] = `${element[0]} (${element[1]})`;
    });

    const title = `Origem (${(<any>grid.datasource).filteredData.length})`;
    const name = 'Origin';
    const id = 'graphOrigin';

    const params = {
      datasource,
      title,
      name,
      id,
    };

    Graph.createGraphPie(params);
  }

  public createGraphModality() {
    const grid = this.getGrid();

    const datasource: any[] = Object.entries((<any>grid.datasource).filteredData.reduce((acc: { [x: string]: number; },
      cv: { TOGO: string | number; }) => {
      acc[cv.TOGO] = acc[cv.TOGO] || 0;
      acc[cv.TOGO] += 1;
      return acc;
    }, {}));

    datasource.forEach((element: any) => {
      element[0] = `${element[0]} (${element[1]})`;
    });

    const title = `Modalidade (${(<any>grid.datasource).filteredData.length})`;
    const name = 'Modality';
    const id = 'graphModality';

    const params = {
      datasource,
      title,
      name,
      id,
    };

    Graph.createGraphPie(params);
  }

  public createGraphCancel() {
    const grid = this.getGrid();
    const datasource: any[] = Object.entries((<any>grid.datasource).filteredData.reduce((acc: { [x: string]: number; },
      cv: { IDSTCOMANDA: string | number; }) => {
      if (cv.IDSTCOMANDA === 'cancelado') {
        acc.Cancelados = acc.Cancelados || 0;
        acc.Cancelados += 1;
      } else {
        acc.Ativos = acc.Ativos || 0;
        acc.Ativos += 1;
      }
      return acc;
    }, {}));

    datasource.sort((a) => {
      if (a[0] === 'Cancelados') {
        return 1;
      }

      return -1;
    });

    datasource.forEach((element: any) => {
      element[0] = `${element[0]} (${element[1]})`;
    });

    const title = `Status (${(<any>grid.datasource).filteredData.length})`;
    const name = 'Status';
    const id = 'graphCancel';
    const color = ['green', 'red'];

    const params = {
      datasource,
      title,
      name,
      id,
      color,
    };

    Graph.createGraphPie(params);
  }

  private async waitForGridCreation(gridName: string): Promise<TekGrid> {
    return new Promise((resolve) => {
      const interval = setInterval(() => {
        const grid = Metadata.getInstance<TekGrid>(gridName);
        if (grid) {
          clearInterval(interval);
          resolve(grid);
        }
      }, 100);
    });
  }

  public async beforeTabChange({ component, tabIndex }: ITabsBeforeChangeEventParam): Promise<String> {
    this.activeTab = component.tabs[tabIndex].name;
    return this.activeTab;
  }

  public async tabChange(): Promise<void> {
    try {
      const gridName = this.activeTab === 'orders-tab' ? 'main-page-grid' : 'scheduled-orders-grid';
      await this.waitForGridCreation(gridName);
      const tooltip: any = this.getGrid()?.toolbarSlot.find((element: any) => element.name === 'tooltip-start-and-stop');
      if (this.currentInterval > 0) this.stopRequest(tooltip);

      if (this.firstTime) {
        this.historyToday();
        this.firstTime = false;
      }
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public startAndStopRequest(): void {
    const grid = this.getGrid();
    const tooltip: any = grid.toolbarSlot.find((element: any) => element.name === 'tooltip-start-and-stop');
    const automaticSearchInterval = ms('60s');

    if (this.currentInterval === 0) {
      this.currentInterval = setInterval(() => {
        this.startRequest(grid);
      }, automaticSearchInterval);
      alertService('success', 'Iniciando busca de pedidos automática...', 'green');
      this.updateInterface(tooltip, 'mdi-pause-circle-outline', 'Pausar busca de pedidos');
    } else {
      this.stopRequest(tooltip);
    }
  }

  public async startRequest({ appliedFilter }: any): Promise<void> {
    await (appliedFilter ? this.executeStartFilter(appliedFilter) : this.historyToday(appliedFilter));
  }

  public stopRequest(tooltip: Tooltip): void {
    clearInterval(this.currentInterval);
    alertService('success', 'Pausando busca de pedidos automática...', 'green');
    this.currentInterval = 0;
    this.updateInterface(tooltip, 'mdi-play-circle-outline', 'Iniciar busca de pedidos');
  }

  public updateInterface(tooltip: Tooltip, iconName: string, label: string): void {
    tooltip.label = label;
    const button = this.getChildElement(tooltip, 'button-start-and-stop');
    if (button) button.iconName = iconName;
  }

  public async openOrderDetailsModal(row: Comanda): Promise<void> {
    if (this.persistentModal) this.persistentModal.destroy();
    this.persistentModal = await ModalService.createFromJson('main-page-persistent-modal', true);

    this.persistentModal.show();
    await this.setFormPersistentDetails(row);
    this.setAtecipateVisible();
  }

  public async getInstance<T extends Instances>(name: string): Promise<T> {
    const instance: T = await new Promise((resolve) => {
      setTimeout(() => {
        resolve(Metadata.getInstances(name)[0]);
      }, 0);
    });
    return instance;
  }

  public formatStatus(value: string): string {
    const commandStatusMap: { [key: string]: string } = {
      1: 'Em produção',
      2: 'Em produção',
      3: 'Em produção',
      4: 'Cancelado',
      5: 'Expedido',
      6: 'Não pago',
      X: 'Finalizado',
      P: 'Impresso',
      A: 'Agendado',
      N: 'Não sincronizado',
    };
    return commandStatusMap[value];
  }

  public setIndexTab({ component, tabIndex }: ITabsBeforeChangeEventParam): void {
    switch (component.tabs[tabIndex].name) {
      case 'delivery-address':
        this.tabDeliveryAddress();
        break;
      case 'order-items':
        this.tabOrderItems();
        break;
      default:
        this.tabPaymentMethods();
    }
  }

  public async getParamsPersistentDetailsModal(): Promise<ParamsRequestPersistentDetails> {
    const form = await this.getInstance<Form>('form-details');
    const { NRCOMANDA, CDFILIAL, NRVENDAREST } = form.value;
    return { NRCOMANDA, CDFILIAL, NRVENDAREST };
  }

  public async setFormPersistentDetails(row: Comanda): Promise<void> {
    const form = await this.getInstance<Form>('form-details');
    row.STATUS = this.formatStatus(row.STATUS);
    row.NMENTREGADOR = row.NMENTREGADOR === null ? 'Produto não liberado' : row.NMENTREGADOR;

    const url = '/order/details';
    const { data } = await this.requestTabsPersistentModal(url, { CDOPERADOR: row.CDOPERADOR });

    const infoDetailsPersistent = { ...row, ...data };
    if (form) form.value = infoDetailsPersistent;
  }

  public async tabDeliveryAddress(): Promise<void> {
    const formAddress = await this.getInstance<Form>('form-address');
    const { NRCOMANDA, CDFILIAL } = await this.getParamsPersistentDetailsModal();
    const url = '/order/address';
    const { data } = await this.requestTabsPersistentModal(url, { NRCOMANDA, CDFILIAL });

    const address = data.reduce((item: Address) => item);
    if (formAddress) formAddress.value = address;
  }

  public async tabOrderItems(): Promise<void> {
    const grid = await this.getInstance<TekTreeGrid>('order-items-grid');
    const { NRCOMANDA, CDFILIAL, NRVENDAREST } = await this.getParamsPersistentDetailsModal();
    const url = '/order/itemsRepo';
    const { data } = await this.requestTabsPersistentModal(url, { NRCOMANDA, CDFILIAL, NRVENDAREST });

    if (grid instanceof TekTreeGrid && data) {
      await this.setRequestTabOrderItems(grid, data, { NRCOMANDA, CDFILIAL, NRVENDAREST });
    }
  }

  public async setRequestTabOrderItems(grid: TekTreeGrid,
    response: RequestOrderItemsRepo[],
    params: ParamsRequestPersistentDetails): Promise<void> {
    const urlSubItens = '/order/subItensRepo';
    const foudItem = response.find((element: RequestOrderItemsRepo) => element.ISCOMBO === true);

    if (foudItem) {
      const { CDPRODPROMOCAO, NRSEQPRODCOM, CDPRODUTO } = foudItem;
      const paramsSubItens = { ...params, CDPRODPROMOCAO, NRSEQPRODCOM };
      const { data } = await this.requestTabsPersistentModal(urlSubItens, paramsSubItens);

      const subItens = data.map((element: RequestOrderItemsRepoSub) => {
        if (NRSEQPRODCOM === element.NRSEQPRODCOM) {
          element.CHILDRENGROUPED = CDPRODUTO;
        }
        return element;
      });

      const joinArrayItens = [...response, ...subItens];
      grid.datasource.updateData(joinArrayItens);
    } else {
      grid.datasource.updateData(response);
    }
  }

  public async tabPaymentMethods(): Promise<void> {
    const gridPayment = await this.getInstance<TekGrid>('payment-methods-grid');
    const { NRVENDAREST, CDFILIAL } = await this.getParamsPersistentDetailsModal();
    const url = '/order/paymentMethods';
    const { data } = await this.requestTabsPersistentModal(url, { NRVENDAREST, CDFILIAL });

    if (gridPayment) gridPayment.datasource.updateData(data);
  }

  public async requestTabsPersistentModal(url: string, params: any): Promise<any> {
    try {
      const { data: { data, error, message } } = await Http.post(url, params);
      return { data, error, message };
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
    return { data: null, error: null, message: null };
  }

  public async openPersistentDetailsItemModal({ row }: RowOpenPersistentDetailsItem): Promise<void> {
    if (row.ISCOMBO) return;
    if (this.persistentDetailsItemModal) this.persistentDetailsItemModal.destroy();
    this.persistentDetailsItemModal = await ModalService.createFromJson('main-page-persistent-details-item-modal', true);

    this.persistentDetailsItemModal.show();
    const form = await this.getInstance<Form>('form-details-item');
    if (form) form.value = row;
  }

  public async setIndexTabDetailsItem({ component, tabIndex }: ITabsBeforeChangeEventParam): Promise<void> {
    if (component.tabs[tabIndex].name === 'comments') {
      await this.tabPersistentDetailsObservation();
    }
  }

  public async tabPersistentDetailsObservation(): Promise<void> {
    const formDetails = await this.getInstance<Form>('form-details-item');
    const grid = await this.getInstance<TekGrid>('comments-grid');

    const {
      CDFILIAL, NRCOMANDA, NRVENDAREST, NRPRODCOMVEN,
    } = formDetails.value;

    if (CDFILIAL && NRCOMANDA && NRVENDAREST && NRPRODCOMVEN) {
      const { data } = await this.requestTabsPersistentModal('/order/itemObservation', {
        CDFILIAL, NRCOMANDA, NRVENDAREST, NRPRODCOMVEN,
      });
      if (grid) grid.datasource.updateData(data);
    }
  }

  public setAtecipateVisible() {
    const button = Metadata.getInstance<ZdButton>('main-page-persistent-anticipate-button');
    if (this.activeTab === 'scheduled-orders-tab') {
      button.isVisible = true;
    } else {
      button.isVisible = false;
    }
  }

  public async forwardOrder(): Promise<void> {
    try {
      const formModalForward: any = this.getChildElement(this.modalForward, 'main-page-modal-form-forward');
      const instanceForm = Metadata.getInstance<Form>(formModalForward.name);

      if (!instanceForm.validate()) return;

      const {
        branch, order, orderFos,
      } = instanceForm.value;

      LoadingService.show();

      const { data } = await Http.post('/order/forwardOrder', {
        CDFILIAL: branch,
        NRCOMANDA: order,
        NRVENDAREST: orderFos,
      });

      if (data.error) throw new Error(data.message);
      this.modalForward?.destroy();
      if (this.getGrid().datasource.data.length === 1) {
        this.getGrid().datasource.updateData([]);
      }
      try {
        const paramsFilter = this.formatParamsFilter();
        if (paramsFilter) await this.historyToday();
      } catch (error) {
        await this.historyToday();
      }
      alertService('alert', `${data.message}`, 'success');
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    } finally {
      LoadingService.hide();
    }
  }

  public async openModalForwardOrder({ row }: any) {
    if (this.modalForward) this.closeModalForwardOrder();
    this.modalForward = await ModalService.createFromJson('main-page-modal-forward', true);
    const formModalForward: any = this.getChildElement(this.modalForward, 'main-page-modal-form-forward');
    const branch: any = this.getChildElement(formModalForward, 'form-branch');
    const order: any = this.getChildElement(formModalForward, 'form-order');
    const orderFos: any = this.getChildElement(formModalForward, 'form-order-fos');
    const origin: any = this.getChildElement(formModalForward, 'form-origin');

    branch.text = `<strong>${branch.text}</strong> ${row.CDFILIAL}`;
    branch.value = row.CDFILIAL;

    order.text = `<strong>${order.text}</strong> ${row.NRCOMANDA}`;
    order.value = row.NRCOMANDA;

    orderFos.text = `<strong>${orderFos.text}</strong> ${row.NRVENDAREST}`;
    orderFos.value = row.NRVENDAREST;

    origin.text = `<strong>${origin.text}</strong> ${row.IDORGCMDVENDA}`;
    origin.value = row.IDORGCMDVENDA;

    formModalForward.value = {
      branch: branch.value,
      order: order.value,
      orderFos: orderFos.value,
      origin: origin.value,
    };

    this.modalForward.show();
  }

  public closeModalForwardOrder(): void {
    this.modalForward.destroy();
  }

  public async refreshGrid(): Promise<void> {
    const paramsFilter = this.formatParamsFilter();
    await (paramsFilter ? this.historyToday(paramsFilter) : this.historyToday());

    const grid = this.getGrid();
    const { data } = grid.datasource;

    data.forEach((item: any) => {
      const chip = this.getChipByKey(item.NRCOMANDA);
      if (chip) this.colorStatus({ component: chip });
    });
  }

  public async validateRequestConsumer({ component }: IEventParam<ZdSelect>): Promise<void> {
    const { searchValue } = (<any>component);
    const form = this.formFilter();

    if (searchValue.length >= 4) {
      const { data: { data } } = await Http.get('/order/consumer', { params: { consumers: searchValue } });

      Object.values(form?.value.consumer)
        .filter((item) => item !== searchValue)
        .map((item) => (data.push({ CONSUMIDOR: item })));

      if (data) component.datasource.data = data;
    }
  }
}
